#include <SoftwareSerial.h>
 
#define DEBUG true

#include <DHT.h>
DHT dht(3, DHT11);

SoftwareSerial espSerial(10,11); 

void setup()
{
  Serial.begin(9600);
  espSerial.begin(9600); // your esp's baud rate might be different

  pinMode(3, INPUT);     // DHT11
  dht.begin();
  
  pinMode(13, OUTPUT);
  digitalWrite(13, LOW);
  sendData("AT+RST\r\n",2000,DEBUG); // reset module
  sendData("AT+CWMODE=1\r\n",1000,DEBUG); // configure standard node
  sendData("AT+CWJAP=ssid,pwd\r\n",5000,DEBUG);
  sendData("AT+CIFSR\r\n",1000,DEBUG); // get ip address
  sendData("AT+CIPMUX=1\r\n",1000,DEBUG); // configure for multiple connections
  sendData("AT+CIPSERVER=1,80\r\n",1000,DEBUG); // turn on server on port 80
  digitalWrite(13, HIGH);
}
 
void loop()
{
//  sendData("AT+CIPSTATUS\r\n",1000, DEBUG);
//  sendData("AT+CIPCLOSE=0\r\n",1000, DEBUG);
//  sendData("AT+CIPCLOSE=1\r\n",1000, DEBUG);
  
  if(espSerial.available()) // check if the esp is sending a message 
  {
    
    if(espSerial.find("+IPD,"))
    {
     delay(1000);
 
     int connectionId = espSerial.read()-48; // subtract 48 because the read() function returns 
                                           // the ASCII decimal value and 0 (the first decimal number) starts at 48
     
     Serial.println(connectionId);
     
     float h = dht.readHumidity();
     float t = dht.readTemperature();

     String webpage = "<html><head><title>ESP8266 web server</title></head><body><h1>Hello world!</h1>";
     webpage += "<br>Temperature: ";
     webpage += t;
     webpage += "<br>Humidity: ";
     webpage += h;
     webpage += "</body></html>\r\n";
 
     String cipSend = "AT+CIPSEND=";
     cipSend += connectionId;
     cipSend += ",";
     cipSend +=webpage.length();
     cipSend +="\r\n";
     
     sendData(cipSend,1000,DEBUG);
     sendData(webpage,3000,DEBUG);
     
     delay(1000);
     String closeCommand = "AT+CIPCLOSE="; 
     closeCommand+=connectionId; // append connection id
     closeCommand+="\r\n";
     
     sendData(closeCommand,1000,DEBUG); 
     delay(1000);   
    }
  }
  sendData("AT+CIPSTATUS\r\n",1000, DEBUG);
}
 
 
void sendData(String command, const int timeout, boolean debug)
{
    
    espSerial.print(command); // send the read character to the esp8266
    
    long int time = millis();
    
    while( (millis() - time) < timeout)
    {
      while(espSerial.available())
      { 
        char c = espSerial.read(); // read the next character.
        Serial.print(c);
      }  
    }
}

