
// based on seeedstudio's example for ESP8266
// Tom Tobback feb 2015
// update firmware to be able to set baud rate to 9600 -- new batch of ESP8266 from TAOBAO have baud rate set to 9600 and firmware 0.9.2.4
// then can use software serial to handle ESP8266 at 9600 baud
// include hardware reset before starting ESP
// software serial on pin 10=UTXD and 11=URXD
// esp reset on pin 9=RST (pull low to reset)
// dht11 on pin 3

#include <SoftwareSerial.h>
#define SSID "xxx"
#define PASS "xxx"
#define THINGSPEAK_IP "184.106.153.149"    // thingspeak.com
SoftwareSerial espSerial(10, 11);          // RX, TX

#include <DHT.h>
DHT dht(3, DHT11);

unsigned long sample_interval = 5000;
unsigned long start_time;

void setup()
{
 pinMode(10, INPUT);    // softserial RX
 pinMode(11, OUTPUT);   // softserial TX
 pinMode(9, OUTPUT);    // esp reset
 pinMode(3, INPUT);     // DHT11
 
 delay(3000);           // to enable uploading
 
 Serial.begin(9600);    // for Serial Monitor
 espSerial.begin(9600); // for ESP, can't be faster than 19200 for softserial
 dht.begin();

 espSerial.setTimeout(2000);

 start_time = millis();  //get the current time;
 Serial.println("DHT11 sensor sending data to THINGSPEAK.COM by ESP8266");
 Serial.print("sample interval (s): ");
 Serial.println(sample_interval/1000);
 Serial.println();

 }

void loop()
{
    if ((millis()-start_time) > sample_interval)  
    {
        float h = dht.readHumidity();
        float t = dht.readTemperature();

        Serial.print(t,2);
        Serial.print("degC");
        Serial.print("\t");
        Serial.print(h,2);
        Serial.print("% \t");
          
        sendDataThingSpeak(t,h);
  
        start_time = millis();
    }
}

//////////////////////////////////////  THINGSPEAK.COM //////////////////////////////////////////////////////

boolean sendDataThingSpeak(float temp, float hum)  
{
   Serial.println(F("reset ESP8266.."));    //test if the module is ready
 
   digitalWrite(9, LOW);     // reset the ESP module
   delay(100);
   digitalWrite(9, HIGH);
  
   espSerial.println("AT+RST");
   delay(1000); 

   while (espSerial.available()) {
   char c = espSerial.read();
   Serial.write(c);
   }

   // set mode to node
   espSerial.println("AT+CWMODE=1");
   delay(1000);
   
   while (espSerial.available()) {
   char c = espSerial.read();
   Serial.write(c);
   }
   
   // connect to wifi
   String cmd="AT+CWJAP=\"";
   cmd+=SSID;
   cmd+="\",\"";
   cmd+=PASS;
   cmd+="\"";
   espSerial.println(cmd);
   delay(5000);
 
   while (espSerial.available()) {
   char c = espSerial.read();
   Serial.write(c);
   }
 
   //set the single connection mode
   espSerial.println("AT+CIPMUX=0");
   delay(1000);
 
   while (espSerial.available()) {
   char c = espSerial.read();
   Serial.write(c);
   }
 
   // connect to TCP socket
   cmd = "AT+CIPSTART=\"TCP\",\"";
   cmd += THINGSPEAK_IP;
   cmd += "\",80";
   espSerial.println(cmd);
   delay(1000);
   
   while (espSerial.available()) {
   char c = espSerial.read();
   Serial.write(c);
   }

   // send HTTP request
   cmd = "GET /update?key=xxx&field1=";
   cmd += temp;
   cmd += "&field2=";
   cmd += hum;
   cmd += "\r\n";

   espSerial.print("AT+CIPSEND=");
   espSerial.println(cmd.length());
 
   while (espSerial.available()) {
   char c = espSerial.read();
   Serial.write(c);
   }
      
   espSerial.println(cmd);
   delay(1000);
  
   while (espSerial.available()) {
   char c = espSerial.read();
   Serial.write(c);
   }
}



