
// based on seeedstudio's example for ESP8266
// Tom Tobback feb 2015
// update firmware to be able to set baud rate to 9600 -- new batch of ESP8266 from TAOBAO have baud rate set to 9600 and firmware 0.9.2.4
// then can use software serial to handle ESP8266 at 9600 baud
// include hardware reset before starting ESP
// software serial on pin 10=UTXD and 11=URXD
// esp reset on pin 9=RST (pull low to reset)
// dht11 on pin 3

#include <SoftwareSerial.h>
#define SSID "xxx"
#define PASS "xxx"
#define EMONCMS_IP "80.243.190.58"         // emoncms.org
#define THINGSPEAK_IP "184.106.153.149"    // thingspeak.com
SoftwareSerial espSerial(10, 11);          // RX, TX

#include <DHT.h>
DHT dht(3, DHT11);

unsigned long sample_interval = 5000;
unsigned long start_time;

void setup()
{
 pinMode(10, INPUT);    // softserial RX
 pinMode(11, OUTPUT);   // softserial TX
 pinMode(9, OUTPUT);    // esp reset
 pinMode(3, INPUT);     // DHT11
 
 delay(3000);           // to enable uploading
 
 Serial.begin(9600);    // for Serial Monitor
 espSerial.begin(9600); // for ESP, can't be faster than 19200 for softserial
 dht.begin();

 espSerial.setTimeout(2000);

 start_time = millis();  //get the current time;
 Serial.println("DHT11 sensor sending data to THINGSPEAK.COM by ESP8266");
 Serial.print("sample interval (s): ");
 Serial.println(sample_interval/1000);
 Serial.println();

 }

void loop()
{
    if ((millis()-start_time) > sample_interval)  
    {
        float h = dht.readHumidity();
        float t = dht.readTemperature();

        Serial.print(t,2);
        Serial.print("degC");
        Serial.print("\t");
        Serial.print(h,2);
        Serial.print("% \t");
        
 /*     boolean sent=false;
        for(int i=0;i<5;i++)
           {
           if(sendDataEmoncms(t,h))
             {
             sent = true;
             break;
             }
           }
        if (!sent){
           Serial.println(F("failed 5 times to send"));
           }
  */
  
        sendDataThingSpeak(t,h);
  
        start_time = millis();
    }
}

///////////////////////////// EMONCMS.ORG ///////////////////////////////////////////////////////////////////////////

boolean sendDataEmoncms(float temp, float hum)  
{
   Serial.print(F("reset ESP8266.."));    //test if the module is ready
 
   digitalWrite(9, LOW);     // reset the ESP module
   delay(100);
   digitalWrite(9, HIGH);
  
   espSerial.println("AT+RST");
   delay(500); 
 
   if(espSerial.find("ready"))     // on TAOBAO batch this is 'ready' instead of 'Ready'
     {
     Serial.print(F("module ready.."));
     }
     else
       {
       Serial.println(F("module no response."));
       return false;
       }
 
// espSerial.println("AT+GMR");
// Serial.println(espSerial.readString());

 //delay(1000);
 //connect to the wifi
   boolean connected=false;

   for(int i=0;i<5;i++)
   {
   if(connectWiFi())
     {
     connected = true;
     break;
     }
   }
   if (!connected){
     Serial.println(F("tried 5 times to connect, please reset"));
     return false;
     }
   delay(500);
  
 //set the single connection mode
   espSerial.println("AT+CIPMUX=0");
   delay(500);
 
   espSerial.flush();
 
   String cmd; 
   cmd = "AT+CIPSTART=\"TCP\",\"";
   cmd += EMONCMS_IP;
   cmd += "\",80";
   espSerial.println(cmd);
// Serial.println(cmd);
 
   if(espSerial.find("Error")) return false;

   String action;
   action = "GET /emoncms/input/post.json?node=23&apikey=xxx&csv=";
   action += temp;
   action += ",";
   action += hum;
   action += " HTTP/1.0\r\n\r\n";

   espSerial.print("AT+CIPSEND=");
   espSerial.println(action.length());
 
   if(espSerial.find(">"))
     {
     Serial.print(">");
     } 
     else 
       {
       espSerial.println("AT+CIPCLOSE");
       Serial.println(F("connect timeout"));
       return false;
       }
 
   espSerial.println(action);
// Serial.println(action);
   delay(1000);
 
   if (espSerial.find("SEND OK"))
     {
     Serial.print(F("send emoncms.org OK.."));
     }
   if (espSerial.find("200 OK"))
     {
     Serial.println(F(" receive OK!"));
     tone(3,1000,100);
     return true;
     }
     else {
       return false;
     }
 
/* while (espSerial.available()) {
   char c = espSerial.read();
   Serial.write(c);
   if(c=='\r') Serial.print('\n');
   }
 */
}

//////////////////////////////////////  THINGSPEAK.COM //////////////////////////////////////////////////////

boolean sendDataThingSpeak(float temp, float hum)  
{
   Serial.print(F("reset ESP8266.."));    //test if the module is ready
 
   digitalWrite(9, LOW);     // reset the ESP module
   delay(100);
   digitalWrite(9, HIGH);
  
   espSerial.println("AT+RST");
   delay(500); 

   /* while (espSerial.available()) {
   char c = espSerial.read();
   Serial.write(c);
   } */

   if(espSerial.find("ready"))   // on TAOBAO batch this is 'ready' instead of 'Ready'
     {
     Serial.print(F("module ready.."));
     }
     else
       {
       Serial.println(F("module no response."));
       return false;
       }
 
 //delay(1000);
 //connect to the wifi
   boolean connected=false;

   for(int i=0;i<5;i++)
   {
   if(connectWiFi())
     {
     connected = true;
     break;
     }
   }
   if (!connected){
     Serial.println(F("tried 5 times to connect, please reset"));
     return false;
     }
   delay(500);
  
 //set the single connection mode
   espSerial.println("AT+CIPMUX=0");
   delay(500);
 
//   espSerial.flush();
 
   String cmd; 
   cmd = "AT+CIPSTART=\"TCP\",\"";
   cmd += THINGSPEAK_IP;
   cmd += "\",80";
   espSerial.println(cmd);
// Serial.println(cmd);
 
   if(espSerial.find("Error")) return false;

   cmd = "GET /update?key=xxx&field1=";
   cmd += temp;
   cmd += "&field2=";
   cmd += hum;
   cmd += "\r\n";

   espSerial.print("AT+CIPSEND=");
   espSerial.println(cmd.length());
 
   if(espSerial.find(">"))
     {
     Serial.print(">");
     } 
     else 
       {
       espSerial.println("AT+CIPCLOSE");
       Serial.println(F("connect timeout"));
       return false;
       }
 
   espSerial.println(cmd);
 //  Serial.println(cmd);
   delay(1000);
 
   if (espSerial.find("SEND OK"))
     {
     Serial.print(F("send thingspeak.com OK.."));
     }
 
 /* while (espSerial.available()) {
   char c = espSerial.read();
   Serial.write(c);
   if(c=='\r') Serial.print('\n');
   }
 */
 
   if (espSerial.find("OK"))
     {
     Serial.println(F(" receive OK!"));
     tone(3,1000,100);
     return true;
     }
     else {
       return false;
     }
}


boolean connectWiFi() {
   espSerial.println("AT+CWMODE=1");
   delay(2000);
   String cmd="AT+CWJAP=\"";
   cmd+=SSID;
   cmd+="\",\"";
   cmd+=PASS;
   cmd+="\"";
// Serial.println(cmd);
   espSerial.println(cmd);
   delay(5000);
 
   if(espSerial.find("OK")) {
     Serial.print(F("connected to wifi.."));
     return true;
     } 
     else 
       {
       Serial.println(F("cannot connect to wifi"));
       return false;
       }
}

